#!/usr/bin/ruby
#encoding: UTF-8

=begin
/******************************************************************************
 *   Copyright © 2017-2021, Michael Uplawski <michael.uplawski@uplawski.eu>   *
 *                                                                            *
 *   This program is free software; you can redistribute it and/or modify     *
 *   it under the terms of the GNU General Public License as published by     *
 *   the Free Software Foundation; either version 3 of the License, or        *
 *   (at your option) any later version.                                      *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   You should have received a copy of the GNU General Public License        *
 *   along with this program; if not, write to the                            *
 *   Free Software Foundation, Inc.,                                          *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                *
 ******************************************************************************/
=end

require_relative 'argparser'
require_relative 'adder'
require_relative 'color_output'
require_relative 'extstring'
require_relative 'user_input'
require_relative 'busy_indicator'
require_relative 'event'
require_relative 'logging'

require 'date'

# Main program class. All action is within the initialize() method, or, more
# precisely in the start_game() method which has to be defined by an
# interface-module. 
class Timequiz
  self::extend(Logging)
  @@log = self::init_logger()

  # Find out, how the program has been called.
  # In juin 2017 this can be 'timequiz' or 'timequizGtk'.
  # Write a Qt-interface, create a symbolic link 'timequizQt' in
  # ../../bin and add a when-clause to the case-when structure, right below the
  # method (in class-scope).
  # Returns true if the given executable file is named the same as the one 
  # called, false if they are not the same.
  # Returns the name of the called executable, if no argument is given.
  def self.called_as?(executable = nil)
    return File::basename($0) == executable.strip if executable
    return File::basename($0) 
  end

  # This adds all the functionality of the named interface module to the
  # current class.
  case called_as?
    # GTK-Interface
  when 'timequizGtk'
    # require_relative 'gtk/timequizGtk'
    # include TimequizGtk
    @log.info('I am sorry. But there is not Gtk-interface, for the time.')
    # Text only
  when 'timequiz'
    require_relative 'console'
    include Console 
    # EXAMPLE
    # when 'timequizSWT'
    #   require_relative 'swt'
    #   include SWT
  else
    # beats me
    # puts "How did you do that?"
    # ... HEY! I guess: You created a link to the executable but forgot
    # something else. Read the comment to the 'called_as?' method, above.
    @log.error(": A user-interface \"" << called_as? << "\" has not (yet) been defined (read the code, my friend)! Aborting.") 
    exit false
  end 

  def verify_prepare(file)
    if(file && !file.empty? && !file.end_with?('.rb'))
       file << ".rb"
    end
    begin
      ofl = File.open(file, 'a')
      if !File.readable?(file)
        @log.error('The file ' << file << ' cannot be read! Aborting')
        exit false
      end
      if(File.empty?(file)  )
        @log.info("The file " << file << " is empty. You must add some events to it.")
        if(File.writable?(file) )
          orig_file = File.dirname(__FILE__) << File::Separator << 'events.rb'
          @log.debug('orig_file is ' << orig_file)
          File.open(orig_file, 'r') do |ifl|
            ofl.puts("# Events defined for the Timequiz game")
            ofl.puts("# Lines starting with '#' are comments.\n\n# EXAMPLES:")
            3.times do 
              line = ""
              line = ifl.readline.strip until line.start_with?('$')
              ofl.puts('#' << line)
            end
            ofl.puts("\n# Add your own events below this line\n#" << '_' * 60)
          end
          ofl.close
        end
        add_ok = require file 
      else
        add_ok = require file 
        if(add_ok)
          return add_ok, $events.length >= 3
        end
      end
    rescue IOError => ex
      ofl.close
      @log.error('Cannot work with the given file: ' << ex.message)
      exit false
    end
  end

  def initialize(*args)
    @log = @@log
    # adjust log-level, if given.
    # Else use what you know.
    options = ArgParser.parse(*args)
    $LOG_LEVEL = Logger::DEBUG if options.debug
    @log.level = $LOG_LEVEL if $LOG_LEVEL 
    @log.debug('log_level is ' << $LOG_LEVEL.to_s)
    @log.debug('options are ' << options.to_s)
    #    should we add only a new event to the list  
    @log.debug('options are : ' << options.to_s)
    add_ok = game_ok = false
    if(options.file)
      @log.debug('shall use non-standard events file')
      add_ok, game_ok = verify_prepare(options.file)
    else
      game_ok = require_relative 'events'
    end
    if(options.add)
      if(add_ok)
        Adder::add(options)
        exit true
      elsif(options.file)
        @log.error('cannot add events to ' << options.file)
        exit false
      else
        @log.error('PSE start the program with -h or --help to see an option-overview')
        exit false
      end
    end

    # start the chosen interface
    if(game_ok)
      start_game 
    else
      @log.error("Cannot play with the events in " << options.file)
      @log.error("PSE verify that there are at least 3 events defined!") 
      exit false
    end
  end

  private

  # A dangerous method. Not only because the use of the argument is somewhat
  # esotheric...  This sorts the events in the *correct* order of their years.
  # Any later comparison with user-provided values is based on the *correct*
  # order, not the one currently at display! Although this is of interest only
  # after the very first round, it caused me some headache, once the
  # user-interfaces had been separated... Mind this when creating a new UI.
  # The argument 'index' controlls if the displayed index should be changed to
  # mirror the *correct* order of events, meaning the index from the Array + 1
  # (i.e. [1,2,3...]). I forgot, how this is useful (see in 'good_order',
  # below).
  def sort_years(index = false)
    @m_events.sort! {|f, s| f.year <=> s.year } 
    @m_events.each_with_index {|ev, i| ev.disp_index = i+1} if index
  end

  # Verifies and reports if the user-provided order is correct in the way that
  # it creates a consistent timeline. The argument 'response' may be either an
  # Array of Integers or just one Integer. In the latter case, an Event should
  # be given, too.
  def good_order(response, event = nil)
    if(response.respond_to?(:to_ary) )
      response_years = []
      # response.each_with_index {|r, i| response_years[r-1] = @m_events[i].year}
      response.each_with_index {|r, i| response_years[i] = @m_events[r-1].year}
      @log.debug('response is ' << response.join(', ') << "\nresponse_years is : " << response_years.join(', ') )
      puts 'response is ' << response.join(', ') << "\nresponse_years is : " << response_years.join(', ') 

      # do not change the displayed index
      sort_years(false)
      event_years = @m_events.collect{|ev| ev.year }

      # Compare the order of years, not the order of events!
      # There may be two or more events in the same year.
      response_years == event_years
    elsif(response.respond_to?(:to_int) )
      # set/change the displayed index 
      sort_years(true)
      # Compare the order of years, not the order of events!
      # There may be two or more events in the same year.
      @m_events[response + 1].year == event.year
    end
  end


end


## ---TEST
if __FILE__ == $0
  qu = Timequiz.new
end

## ---END TEST
# EOF, too.
